/**
  *  \file planetscentral/dbclient/dbsort.h
  *  \brief Database Client: Sorting
  */
#ifndef C2SERVER_PLANETSCENTRAL_DBCLIENT_DBSORT_H
#define C2SERVER_PLANETSCENTRAL_DBCLIENT_DBSORT_H

#include "planetscentral/dbclient/dbclient.h"
#include "cpluslib/smartptr.h"

class DbKey;
class DbHashField;
class DbList;

/** Sorting. This class is used to build SORT commands.
    - call a key's sort() function to get a DbSort object
    - call by(), get(), etc. to build the SORT request
    - call getResult() to get the result, or storeResult() to store it somewhere */
class DbSort {
 public:
    DbSort(DbClient& client, const string_t& keyName);
    ~DbSort();

    DbSort& by(string_t pattern);
    DbSort& by(const DbKey& key);
    DbSort& by(const DbHashField& key);
    DbSort& get();
    DbSort& get(string_t pattern);
    DbSort& get(const DbKey& key);
    DbSort& get(const DbHashField& key);
    DbSort& limit(int32_t start, int32_t count);
    DbSort& sortLexicographical();
    DbSort& sortReversed();
    DbSort& dontSort();

    void      getResult(StringList_t& out);
    void      getResult(IntList_t& out);
    IntValue* getResult();
    void      storeResult(DbList out);

 private:
    DbClient& client;
    Ptr<DbRequest> req;
};

#endif
