/**
  *  \file planetscentral/talk/text.cc
  *  \brief Talk: Representation of Text
  */

#include "planetscentral/talk/text.h"

/** To be called on list nodes: check for simple list. A simple list is one
    where every list item has just one paragraph as content. */
bool
TextNode::isSimpleList() const
{
    // Is this a list?
    if (major != maGroup || minor != miGroupList) {
        return false;
    }

    // Check list items
    for (ptr_vector<TextNode>::size_type i = 0, n = children.size(); i != n; ++i) {
        if (children[i]->major != maGroup || children[i]->minor != miGroupListItem) {
            return false;
        }
        if (children[i]->children.size() != 1) {
            return false;
        }

        TextNode* c = children[i]->children[0];
        if (c->major != maParagraph || c->minor != miParNormal) {
            return false;
        }
    }

    // No violation found
    return true;
}

/** Remove all quotes from this text. Updates the node in-place. */
void
TextNode::stripQuotes()
{
    // Sort all quotes to the front
    ptr_vector<TextNode>::size_type in = 0, out = 0, n = children.size();
    while (in < n) {
        if (children[in]->major == maGroup && children[in]->minor == miGroupQuote) {
            // it's a quote, drop it
        } else {
            // not a quote
            children.swap_elems(in, out);
            ++out;
        }
        ++in;
    }

    // Now, all quotes are after position /out/. Delete them.
    while (children.size() > out) {
        children.pop_back();
    }
}

/** Get content of this node as raw text. */
string_t
TextNode::rawTextContent()
{
    if (major == maPlain) {
        return text;
    } else {
        string_t result;
        for (ptr_vector<TextNode>::size_type i = 0, n = children.size(); i < n && result.size() < 10000; ++i) {
            result += children[i]->rawTextContent();
        }
        return result;
    }
}
